package cn.com.ty.lift.quan.news.service.impl;

import cn.com.ty.lift.quan.news.dao.entity.MapInfo;
import cn.com.ty.lift.quan.news.dao.entity.QuanNews;
import cn.com.ty.lift.quan.news.dao.entity.model.QuanNewsReq;
import cn.com.ty.lift.quan.news.dao.mapper.QuanNewsMapper;
import cn.com.ty.lift.quan.news.service.QuanNewsService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.AllArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 电梯圈新闻公告 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-02
 */
@Service
@AllArgsConstructor
public class QuanNewsServiceImpl extends ServiceImpl<QuanNewsMapper, QuanNews> implements QuanNewsService {

    private final RedisTemplate<String, Object> redisTemplate;

    /**
     * 查询置顶新闻+普通新闻列表
     *
     * @since 2019/12/3 10:30
     */
    public RestResponse appNews(QuanNewsReq request) {
        Page<QuanNews> page = new Page<>(request.getPageNum(), request.getPageSize());
        IPage<Map<String, Object>> news = this.pageMaps(page, new QueryWrapper<QuanNews>()
                .select("id", "title", "content", "top_serial topSerial", "video_url videoUrl", "release_id releaseId", "release_user releaseUser", "release_date releaseDate")
                .like(null != request.getTitle(), "title", request.getTitle())
                .eq("top_serial", "0")
                .orderByDesc("release_date"));
        //全部新闻的浏览量列表
        Map<Object, Object> lookTimes = redisTemplate.boundHashOps("system-quanNews-look-time").entries();
        //全部新闻的点赞用户列表
        Map<Object, Object> likeUsers = redisTemplate.opsForHash().entries("system-quanNews-like-users");
        HashMap<String, Object> result = new HashMap<>();
        newsAddLookAndLike(news.getRecords(), lookTimes, likeUsers);
        result.put("news", news);
        if (request.getHaveTops() == 1) {
            List<Map<String, Object>> topNews = this.listMaps(new QueryWrapper<QuanNews>()
                    .select("id", "title", "content", "top_serial topSerial", "video_url videoUrl", "release_id releaseId", "release_user releaseUser", "release_date releaseDate")
                    .ne("top_serial", "0")
                    .orderByAsc("top_serial"));
            //把点赞的用户数量和浏览量整入新闻字段中
            newsAddLookAndLike(topNews, lookTimes, likeUsers);
            result.put("topNews", topNews);
        }
        return RestResponse.success(result);
    }

    public RestResponse news(QuanNewsReq request) {
        Page<QuanNews> page = new Page<>(request.getPageNum(), request.getPageSize());
        IPage<Map<String, Object>> news = this.pageMaps(page, new QueryWrapper<QuanNews>()
                .select("id", "title", "content", "top_serial topSerial", "video_url videoUrl", "release_id releaseId", "release_user releaseUser", "release_date releaseDate")
                .orderByDesc("release_date"));
        //全部新闻的浏览量列表
        Map<Object, Object> lookTimes = redisTemplate.boundHashOps("system-quanNews-look-time").entries();
        //全部新闻的点赞用户列表
        Map<Object, Object> likeUsers = redisTemplate.opsForHash().entries("system-quanNews-like-users");
        newsAddLookAndLike(news.getRecords(), lookTimes, likeUsers);
        return RestResponse.success(news);
    }

    /**
     * 将新闻置顶
     *
     * @since 2019/12/3 11:23
     */
    public RestResponse top(Long id, Integer type) {
        QuanNews byId = this.getById(id);
        if (byId == null) {
            return RestResponse.success();
        }
        //1置顶 0取消置顶
        if (type == 1) {
            if (byId.getTopSerial() != 0) {
                //如果新闻在置顶新闻中
                Integer serial = byId.getTopSerial();

                List<QuanNews> tops = this.list(new QueryWrapper<QuanNews>().ne("top_serial", 0));
                for (QuanNews top : tops) {
                    if (top.getTopSerial() < serial) {
                        top.setTopSerial(top.getTopSerial() + 1);
                    }
                }
                byId.setTopSerial(1);
                //更新排序
                this.updateBatchById(tops);
                this.updateById(byId);
            } else {
                //如果新闻不是置顶新闻
                //获取置顶新闻
                List<QuanNews> news = this.list(new QueryWrapper<QuanNews>().select("id", "top_serial").ne("top_serial", 0));
                if (!news.isEmpty()) {
                    //如果有置顶新闻
                    news.forEach(n -> {
                        n.setTopSerial(n.getTopSerial() + 1);
                    });
                    this.updateBatchById(news, 100);
                }
                byId.setTopSerial(1);
                //更新置顶新闻列表，新闻
                if (this.updateById(byId)) {
                    return RestResponse.success(null, "置顶成功");
                } else {
                    return RestResponse.fail();
                }
            }
            return RestResponse.success(null, "置顶成功");
        }
        //1置顶 0取消置顶
        if (type == 0) {
            if (byId.getTopSerial() == 0) {
                //如果不是置顶新闻
                return RestResponse.success(null, "不是置顶新闻");
            }
            List<QuanNews> tops = this.list(new QueryWrapper<QuanNews>().select("id", "top_serial").ne("top_serial", 0).ne("id", id));
            //如果置顶新闻为0个
            if (tops.isEmpty()) {
                byId.setTopSerial(0);
                if (this.updateById(byId)) {
                    return RestResponse.success(null, "取消置顶成功");
                } else {
                    return RestResponse.fail();
                }
            }
            //比这个新闻排序大的都 序号-1
            tops.forEach(top -> {
                if (top.getTopSerial() > byId.getTopSerial()) {
                    top.setTopSerial(top.getTopSerial() - 1);
                }
            });
            byId.setTopSerial(0);
            if (this.updateById(byId) && this.updateBatchById(tops)) {
                return RestResponse.success(null, "取消置顶成功");
            } else {
                return RestResponse.fail();
            }
        }
        return RestResponse.failParam();
    }

    //把点赞的用户数量和浏览量整入新闻字段中
    void newsAddLookAndLike(List<Map<String, Object>> records, Map<Object, Object> lookTimes, Map<Object, Object> likeUsers) {
        for (Map<String, Object> record : records) {
            Object id = record.get("id");
            //如果没有浏览量
            if (lookTimes.isEmpty()) {
                record.put("lookNum", 0);
            } else {
                //遍历lookTimes，查看对应id的新闻浏览量
                for (Map.Entry<Object, Object> entry : lookTimes.entrySet()) {
                    Object key = entry.getKey();
                    Object value = entry.getValue();
                    if (key.equals(id.toString())) {
                        record.put("lookNum", value);
                        break;
                    }
                    record.put("lookNum", 0);
                }
            }
            //如果没有用户喜爱
            if (likeUsers.isEmpty()) {
                record.put("likeNum", 0);
            } else {
                //遍历likeUsers，查看对应id的新闻的点赞量
                for (Map.Entry<Object, Object> entry : likeUsers.entrySet()) {
                    String k = entry.getKey().toString();
                    ArrayList v = (ArrayList) entry.getValue();
                    if (k.equals(id.toString())) {
                        record.put("likeNum", v.size());
                        break;
                    }
                    record.put("likeNum", 0);
                }
            }
        }
    }


    public RestResponse like(QuanNewsReq request) {
        String userId = request.getUserId().toString();
        String id = request.getId().toString();
        ArrayList users = (ArrayList) redisTemplate.opsForHash().get("system-quanNews-like-users", id);
        if (null == users) {
            users = new ArrayList<>();
        } else {
            //遍历userId列表
            for (Object user : users) {
                if (user.equals(userId)) {
                    //如果列表内有这个uid就删除喜欢,并return
                    users.remove(user);
                    redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
                    return RestResponse.success(null, "取消点赞成功");
                }
            }
        }
        //如果没有列表或列表内没有这个uid就标记喜欢
        users.add(userId);
        redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
        return RestResponse.success(null, "点赞成功");
    }

    //获取一条新闻
    public RestResponse getQuanNews(QuanNewsReq request) {
        Map<String, Object> byId = this.getMap(new QueryWrapper<QuanNews>().eq("id", request.getId()));
        if (null != byId) {
            String id = byId.get("id").toString();
            Long lookNum = redisTemplate.boundHashOps("system-quanNews-look-time").increment(id, 1);
            int isLike;
            ArrayList users = (ArrayList) redisTemplate.opsForHash().get("system-quanNews-like-users", id);
            if (users != null && users.contains(request.getUserId().toString())) {
                isLike = 1;
            } else {
                isLike = 0;
            }
            byId.put("lookNum", lookNum);
            byId.put("isLike", isLike);
            return RestResponse.success(byId, "成功");
        } else {
            return RestResponse.success();
        }
    }

    public RestResponse areas() {
        List<String> areas = this.getBaseMapper().areas();
        if (areas.isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(areas);
    }

    //传入经纬度，返回城市名称，基于百度地图，文档http://lbsyun.baidu.com/index.php?title=uri/api/web#service-page-anchor2
    public RestResponse location(String point) throws IOException {
        ObjectMapper jackson = new ObjectMapper();
        jackson.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        String body = new RestTemplate()
                .getForEntity("http://api.map.baidu.com/geocoder?location=" + point + "&output=json", String.class)
                .getBody();
        MapInfo mapInfo = jackson.readValue(body, MapInfo.class);
        if (mapInfo.getCity() != null) {
            return RestResponse.success(mapInfo.getCity().replace("市", ""));
        }
        return RestResponse.fail();
    }
}